/*
 * LCD display library for SC1602 module (HITACHI HD44780U controller)
 * 4 bit transfer mode
 * lcd4.c
 */
#include "lcd4.h"
#include <xc.h>

/*
 make additional delay if clock is faster than 28MHz
 E >=140ns
*/
#if _XTAL_FREQ > 28571428
  #define STROBE (LCD4_E=1,asm("nop"),LCD4_E=0)
#else
  #define STROBE (LCD4_E=1,LCD4_E=0)
#endif

/*
 LCD initialization
*/
void Init_LCD4(void){
    /* secure 8bit config */
    __delay_ms(15);	// > 15ms after power on
    LCD4_PORT=0x3;	// E=0, RS=0, set 8bit mode(0x30)
    STROBE;
    __delay_ms(5);	// > 4.1ms
    STROBE;
    __delay_us(100);	// > 100us
    STROBE;
    __delay_us(40);	// > 40us
    LCD4_PORT=0x2; 	// E=0, RS=0, set 4bit mode(0x20)
    STROBE;
    __delay_us(40);	// > 40us
    Cmd_LCD4(0x28);	// 4bit, 2line, 5x7 font
    Cmd_LCD4(DISP_ON);	// display on, prompt=off
    Clr_LCD4;
    Cmd_LCD4(ENT_MODE);	// entry mode, cursor=increment, shift=none
}

/*
 write one byte character or command
 c: ASCII code or command
*/
void W_LCD4(unsigned char c, int rs){
    LCD4_RS=rs;                 // set RS
#ifdef _16F84
    LCD4_PORT = c>>4;
    STROBE;
    LCD4_PORT = c&0xf;	// write low data, keep lower 4bits
    STROBE;
#else
    LCD4_PORT = LCD4_PORT&0xf0 + c>>4;
    STROBE;
    LCD4_PORT=LCD4_PORT&0xf0 + c&0xf;	// write low data, keep lower 4bits
    STROBE;
#endif
    __delay_us(40);		// >37us
}

/*
 write character string
 ptr: start pointer of character string which is NULL terminated.
*/
void Puts_LCD4(unsigned char *ptr){
    while(*ptr){
            Putc_LCD4(*ptr++);
    }
}

/*
 shift display to left n times
*/
void Shift_LN(int n){
    int i;
    for (i=0;i<n;i++){
        Shift_L;
    }
}

/*
 shift display to right n times
*/
void Shift_RN(int n){
    int i;
    for (i=0;i<n;i++){
        Shift_R;
    }
}

/*
 move cursor to left n times
*/
void Cursor_LN(int n){
    int i;
    for (i=0;i<n;i++){
        Cursor_L;
    }
}

/*
 move corsor to right n times
*/
void Cursor_RN(int n){
    int i;
    for (i=0;i<n;i++){
        Cursor_R;
    }
}


